function H = calibrateCamera(image)
% Computes and returns homographic matrix H
% Usage : H = calibrateCamera(image)
% H - homographic matrix
% input image
%
% authors : Karel Kohout, FEL CVUT 2003

[u v hodnoty] = impixel(image);

% ------------------------ Transformation ------------------------
% A - matrix with measured data
% H - homographic matrix
% x,y - coordinates of point in a real world
% u,v - coordinates of points in image

% Known real coordinates
x=[200;240;200;240];
y=[ 40; 40; 90; 90];

% Computation matrix of equations
A=[-x(1), -y(1), -1,     0,     0,  0, u(1)*x(1), u(1)*y(1), u(1);
       0,     0,  0, -x(1), -y(1), -1, v(1)*x(1), v(1)*y(1), v(1);
   -x(2), -y(2), -1,     0,     0,  0, u(2)*x(2), u(2)*y(2), u(2);
       0,     0,  0, -x(2), -y(2), -1, v(2)*x(2), v(2)*y(2), v(2);
   -x(3), -y(3), -1,     0,     0,  0, u(3)*x(3), u(3)*y(3), u(3);
       0,     0,  0, -x(3), -y(3), -1, v(3)*x(3), v(3)*y(3), v(3);
   -x(4), -y(4), -1,     0,     0,  0, u(4)*x(4), u(4)*y(4), u(4);
       0,     0,  0, -x(4), -y(4), -1, v(4)*x(4), v(4)*y(4), v(4)];

[U,D,V] = svd(A);% SVD rozklad
Anova = U*D*V';
% an orthonormal basis for the null space of A obtained from the singular value decomposition
H = null(Anova); 
Hres =[ H(1) H(2) H(3);
        H(4) H(5) H(6);
        H(7) H(8) H(9)];
H = Hres;
pause;
close all;
