function Uhly_mnozina=iku(X, G, rg, rf, re)
% inverzni kinematicka uloha
% velka pismena za zacatku oznacuji sloupcove vektory
% X pozice chpadla v absolutnich souradnicich
% G bod G absolutnich souradnicich (viz obrazek)
% rg delka ramena zacinajici v G
% rf delka ramena zacinajici v F
% re delka ramena zacinajici v E
% Uhly_mnozina je [[g1;f1;e1] [g2;f2;e2]] = uhly v rad v kloubech G,F,E

  % X = [3 2 1]';
  % G = [6 5 9]';
  % rg = 1;
  % rf = 7;
  % re = 5;

  X = [X; 1];
  G = [G; 1];
  F = G + [0 0 -rg 0]';

% transformace pocatku do F
  TF = [[eye(3); zeros(1,3)] [-1 -1 -1 1]'.*F];
  XF = TF*X;

% uhel natoceni v lkoubu G, XF(1:2) oznacuje prumet do roviny xy
% pokud je (XF(1:2)' * XF(1:2))==0, pak je inf reseni, XF lezi primo \
% pod G. vyberu napr. 0
  if ((XF(1:2)' * XF(1:2)) == 0)
    g1_velikost = 0;
  else
    g1_velikost = acos(([1 0 0 0] * XF) / (1 * sqrt(XF(1:2)' * XF(1:2))));
  end
  g1_smer = sign(XF(2));
  g1 = g1_velikost * g1_smer;

% jelikoz je dle zadani jedno, kam smeruje predek chapadla
  g2 = -1 * g1_smer * (pi - g1_velikost);

% rotace kolem z tak, aby y souradnice X byla 0 s pouzitim nalezeneho uhlu
  RF1 = [[cos(g1) sin(g1) 0; -sin(g1) cos(g1) 0; [0 0 1]; [0 0 0]] [0 0 0 1]'];
  RF2 = [[cos(g2) sin(g2) 0; -sin(g2) cos(g2) 0; [0 0 1]; [0 0 0]] [0 0 0 1]'];
  XFR1 = RF1*XF;
  XFR2 = RF2*XF;

% vypocet zbylych uhlu
% uhel mezi osou z a XFR1
% muzu si dovolit XFR1(1:3), protoze XFR1(2) je 0
% pohybuji se tedy v rovine x(-z)!
% pokud je (XFR1(1:3)' * XFR1(1:3)) == 0 a parametry robota to
% umoznuji (v teto uloze ne) pak inf reseni. XRF lezi v F
  if ((XFR1(1:3)' * XFR1(1:3)) == 0)
    delta1_velikost = 0;
  else
    delta1_velikost = acos(([0 0 -1 0] * XFR1) / (1 * sqrt(XFR1(1:3)' * XFR1(1:3))));
  end
  delta1_znamenko = sign(XFR1(1));
  delta1 = delta1_velikost * delta1_znamenko;

  if ((XFR2(1:3)' * XFR1(1:3)) == 0)
    delta2_velikost = 0;
  else
    delta2_velikost = acos(([0 0 -1 0] * XFR2) / (1 * sqrt(XFR2(1:3)' * XFR2(1:3))));
  end
  delta2_znamenko = sign(XFR2(1));
  delta2 = delta2_velikost * delta2_znamenko;

  xfr1 = sqrt (XFR1(1:3)' * XFR1(1:3));
  xfr2 = sqrt (XFR2(1:3)' * XFR2(1:3));

  % doufam, ze ramena nejsou nulova
  if (xfr1 == 0)
    beta1 = 0;
    gama1 = 0;
  else
    beta1 = acos ((rf^2 + xfr1^2 - re^2) / (2 * rf * xfr1));
    gama1 = acos ((rf^2 + re^2 - xfr1^2) / (2 * rf * re));
  end

  if (xfr2 == 0)
    beta1 = 0;
    gama1 = 0;
  else
    beta2 = acos ((rf^2 + xfr2^2 - re^2) / (2 * rf * xfr2));
    gama2 = acos ((rf^2 + re^2 - xfr2^2) / (2 * rf * re));
  end

  f1 = delta1 + beta1;
  f2 = delta1 - beta1;
  e1 = gama1 - pi;
  e2 = -e1;

  f3 = delta2 + beta2;
  f4 = delta2 - beta2;
  e3 = gama2 - pi;
  e4 = -e3;

% reseni jsou symtericka, zrcadleni, protoze nezalezi na smeru chapadla
  Uhly_mnozina = [];
  reseni_1 = [g1 f1 e1]';
  reseni_1_imag_exists = cumsum(abs(imag(reseni_1)));
  if (reseni_1_imag_exists(end) == 0)
      Uhly_mnozina = [Uhly_mnozina reseni_1];
  end
  reseni_2 = [g1 f2 e2]';
  reseni_2_imag_exists = cumsum(abs(imag(reseni_2)));
  if (reseni_2_imag_exists(end) == 0)
      Uhly_mnozina = [Uhly_mnozina reseni_2];
  end
  reseni_3 = [g2 f3 e3]';
  reseni_3_imag_exists = cumsum(abs(imag(reseni_3)));
  if (reseni_3_imag_exists(end) == 0)
      Uhly_mnozina = [Uhly_mnozina reseni_3];
  end
  reseni_4 = [g2 f4 e4]';
  reseni_4_imag_exists = cumsum(abs(imag(reseni_4)));
  if (reseni_4_imag_exists(end) == 0)
      Uhly_mnozina = [Uhly_mnozina reseni_4];
  end

%end
